@@ -421,6 +421,9 @@ sub options {
 Accessors that start with a C<_> are marked private -- regular users
 should never need to use these.
 
+See the C<CPANPLUS::Config> documentation for what items can be
+set and retrieved.
+
 =head2 get_SOMETHING( ITEM, [ITEM, ITEM, ... ] );
 
 The C<get_*> style accessors merely retrieves one or more desired
@@ -460,18 +463,10 @@ Locations and names of source files locally.
 
 Locations and names of source files remotely.
 
-=item _set|_get_dist
-
-Mapping of distribution format names to modules.
-
 =item _set|_get_fetch
 
 Special settings pertaining to the fetching of files.
 
-=item _set|_get_daemon
-
-Settings for C<cpanpd>, the CPANPLUS daemon.
-
 =back
 
 =cut
@@ -570,24 +565,25 @@ sub DESTROY { 1 };
 
 =pod
 
+=head1 BUG REPORTS
+
+Please report bugs or other issues to E<lt>bug-cpanplus@rt.cpan.org<gt>.
+
 =head1 AUTHOR
 
-This module by
-Jos Boumans E<lt>kane@cpan.orgE<gt>.
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPAN++ interface (of which this module is a part of) is
-copyright (c) 2001, 2002, 2003, 2004, Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
+The CPAN++ interface (of which this module is a part of) is copyright (c) 
+2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
 
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
+This library is free software; you may redistribute and/or modify it 
+under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<CPANPLUS::Backend>, L<CPANPLUS::Configure::Setup>
+L<CPANPLUS::Backend>, L<CPANPLUS::Configure::Setup>, L<CPANPLUS::Config>
 
 =cut
 
