@@ -1,7 +1,7 @@
 package CPANPLUS::Internals::Extract;
 
 use strict;
-use CPANPLUS::inc;
+
 use CPANPLUS::Error;
 use CPANPLUS::Internals::Constants;
 
@@ -117,7 +117,7 @@ sub _extract {
         prefer_bin  => { default => $conf->get_conf('prefer_bin') },
         extractdir  => { default => $conf->get_conf('extractdir') },
         module      => { required => 1, allow => IS_MODOBJ, store => \$mod },
-        perl        => { default => $conf->get_program('perl') || $^X },
+        perl        => { default => $^X },
     };
     
     my $args = check( $tmpl, \%hash ) or return;
@@ -161,30 +161,68 @@ sub _extract {
         return;
     }
     
+    ### if ->files is not filled, we dont know what the hell was
+    ### extracted.. try to offer a suggestion and bail :(
+    unless ( $ae->files ) {
+        error( loc( "'%1' was not able to determine extracted ".
+                    "files from the archive. Instal '%2' and ensure ".
+                    "it works properly and try again",
+                    $ae->is_zip ? 'Archive::Zip' : 'Archive::Tar' ) );
+        return;                    
+    }                    
+    
+    
     ### print out what files we extracted ###  
     msg(loc("Extracted '%1'",$_),$verbose) for @{$ae->files};  
     
+    ### set them all to be +w for the owner, so we don't get permission
+    ### denied for overwriting files that are just +r
+    
+    ### this is to rigurous -- just change to +w for the owner [cpan #13358] 
+    #chmod 0755, map { File::Spec->rel2abs( File::Spec->catdir($to, $_) ) }
+    #            @{$ae->files};
+    
+    for my $file ( @{$ae->files} ) { 
+        my $path = File::Spec->rel2abs( File::Spec->catfile($to, $file) );
+    
+        $self->_mode_plus_w( file => $path );
+    }
+    
     ### check the return value for the extracted path ###
     ### Make an educated guess if we didn't get an extract_path
     ### back
-    my $dir = File::Spec->rel2abs( 
-                  $ae->extract_path || 
-                  File::Spec->catdir(   $to,
-                                        $mod->package_name .'-'.
-                                        $mod->package_version    
-                                    )
-              );
+    ### XXX apparently some people make their own dists and they 
+    ### pack up '.' which means the leading directory is '.' 
+    ### and only the second directory is the actual module directory
+    ### so, we'll have to check if our educated guess exists first, 
+    ### then see if the extract path works.. and if nothing works...
+    ### well, then we really don't know.
+
+    my $dir;
+    for my $try (
+        File::Spec->rel2abs( 
+            $self->_safe_path( path =>
+                File::Spec->catdir( $to,  
+                                    $mod->package_name .'-'. 
+                                    $mod->package_version 
+        ) ) ),
+        File::Spec->rel2abs( $ae->extract_path ),
+    ) {
+        ($dir = $try) && last if -d $try;
+    }
                                             
     ### test if the dir exists ###
-    unless( -d $dir ) {
+    unless( $dir && -d $dir ) {
         error(loc("Unable to determine extract dir for '%1'",$mod->module));
         return;
     
     } else {    
         msg(loc("Extracted '%1' to '%2'", $mod->module, $dir), $verbose);
         
-        ### register where we extracted the files to ###
+        ### register where we extracted the files to,
+        ### also store what files were extracted
         $mod->status->extract( $dir ); 
+        $mod->status->files( $ae->files );
     }
       
     ### also, figure out what kind of install we're dealing with ###
